-----------------------------------------------------------------------------------------------
-- Client Lua Script for Breakout
-----------------------------------------------------------------------------------------------

require "Window"
require "Unit"
require "GameLib"
require "Sound"

local Breakout = {}

function Breakout:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function Breakout:Init()
    Apollo.RegisterAddon(self)
end

function Breakout:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("Breakout.xml")
	Apollo.LoadSprites("BreakoutSprites.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self) 
end

function Breakout:OnDocumentReady()
	if  self.xmlDoc == nil then
		return
	end
	Apollo.RegisterEventHandler("ActivateCCStateStun", "OnActivateCCStateStun", self) -- Starting the UI
	Apollo.RegisterEventHandler("UpdateCCStateStun", "OnUpdateCCStateStun", self) -- Hitting the interact key
	Apollo.RegisterEventHandler("RemoveCCStateStun", "OnRemoveCCStateStun", self) -- Close the UI
	Apollo.RegisterEventHandler("StunVGPressed", "OnStunVGPressed", self)

	Apollo.RegisterTimerHandler("CalculateTimeRemaining", "OnCalculateTimeRemaining", self)
	
	self.wndButtonUp = Apollo.LoadForm(self.xmlDoc, "ButtonUpWindow", nil, self)
	self.wndButtonDown = Apollo.LoadForm(self.xmlDoc, "ButtonDownWindow", nil, self)
	self.wndButtonLeft = Apollo.LoadForm(self.xmlDoc, "ButtonLeftWindow", nil, self)
	self.wndButtonRight = Apollo.LoadForm(self.xmlDoc, "ButtonRightWindow", nil, self)
	self.wndProgressBar = Apollo.LoadForm(self.xmlDoc, "ProgressBarWindow", nil, self)
	self.wndTimerBar = Apollo.LoadForm(self.xmlDoc, "TimeRemainingWindow", nil, self)
	self:RestoreControlPositions()
	self:ShowControls(false)
	
	Apollo.RegisterSlashCommand("bo", "OnSlashCommand", self)
	Apollo.RegisterSlashCommand("breakout", "OnSlashCommand", self)
	
	if self.displayArrows == nil then
		self.displayArrows = true
	end
	if self.playSounds == nil then
		self.playSounds = true
	end
	
	self.wndProgress = nil
end

function Breakout:OnSlashCommand(slashcommand, subcommand)
	if subcommand == "layout" or subcommand == "l" then
		self:ToggleMode()
	elseif subcommand == "arrow" or subcommand == "arrows" or subcommand == "a" then
		self:ToggleDisplayArrows()
	elseif subcommand == "sound" or subcommand == "sounds" or subcommand == "s" then
		self:ToggleSounds()
	end
end

function Breakout:ToggleMode()
	if self.wndButtonUp and self.wndButtonUp:IsShown() then
		self:ShowControls(false)
	else
		self.wndProgressBar:FindChild("ProgressBar"):SetMax(100)
		self.wndProgressBar:FindChild("ProgressBar"):SetFloor(0)
		self.wndProgressBar:FindChild("ProgressBar"):SetProgress(100)
		self.wndTimerBar:FindChild("TimeRemainingBar"):SetMax(100)
		self.wndTimerBar:FindChild("TimeRemainingBar"):SetProgress(100)
		
		self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):Enable(false)
		self.wndButtonUp:FindChild("ProgressButtonArtUp"):Enable(false)
		self.wndButtonRight:FindChild("ProgressButtonArtRight"):Enable(false)
		self.wndButtonDown:FindChild("ProgressButtonArtDown"):Enable(false)
		
		self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):SetText("")
		self.wndButtonUp:FindChild("ProgressButtonArtUp"):SetText("")
		self.wndButtonRight:FindChild("ProgressButtonArtRight"):SetText("")
		self.wndButtonDown:FindChild("ProgressButtonArtDown"):SetText("")
		
		self:ShowControls(true)
	end
end

function Breakout:ToggleDisplayArrows()
	if self.displayArrows == true then
		self.displayArrows = false;
		Print("Breakout will now display bound keys on buttons")
	elseif self.displayArrows == false then
		self.displayArrows = true
		Print("Breakout will now display directional arrows on buttons")
	end
end

function Breakout:ToggleSounds()
	if self.playSounds == true then
		self.playSounds = false
		Print("Breakout sounds are disabled")
	elseif self.playSounds == false then
		self.playSounds = true
		Print("Breakout sounds are enabled")
	end
end

function Breakout:RestoreControlPositions()
	if self.savedUpLoc then
		self.wndButtonUp:MoveToLocation(self.savedUpLoc)
	else
		self.savedUpLoc = self.wndButtonUp:GetLocation()
	end
	if self.savedDownLoc then
		self.wndButtonDown:MoveToLocation(self.savedDownLoc)
	else
		self.savedDownLoc = self.wndButtonDown:GetLocation()
	end
	if self.savedLeftLoc then
		self.wndButtonLeft:MoveToLocation(self.savedLeftLoc)
	else
		self.savedLeftLoc = self.wndButtonLeft:GetLocation()
	end
	if self.savedRightLoc then
		self.wndButtonRight:MoveToLocation(self.savedRightLoc)
	else
		self.savedRightLoc = self.wndButtonRight:GetLocation()
	end
	if self.savedProgressLoc then
		self.wndProgressBar:MoveToLocation(self.savedProgressLoc)
	else
		self.savedProgressLoc = self.wndProgressBar:GetLocation()
	end
	if self.savedTimerLoc then
		self.wndTimerBar:MoveToLocation(self.savedTimerLoc)
	else
		self.savedTimerLoc = self.wndTimerBar:GetLocation()
	end
end

function Breakout:ShowControls(show)
	self.wndButtonUp:Show(show)
	self.wndButtonDown:Show(show)
	self.wndButtonLeft:Show(show)
	self.wndButtonRight:Show(show)
	self.wndProgressBar:Show(show)
	self.wndTimerBar:Show(show)
	self.wndProgressBar:FindChild("ProgressBar"):Show(show)
	self.wndTimerBar:FindChild("TimeRemainingContainer"):Show(show)
	self.wndTimerBar:FindChild("TimeRemainingBar"):Show(show)
end

-----------------------------------------------------------------------------------------------
-- Rapid Tap
-----------------------------------------------------------------------------------------------

function Breakout:OnActivateCCStateStun(eChosenDirection)
	self:ShowControls(true)
	self.wndTimerBar:FindChild("TimeRemainingContainer"):Show(false)

	local bLeft 	= eChosenDirection == Unit.CodeEnumCCStateStunVictimGameplay.Left
	local bUp 		= eChosenDirection == Unit.CodeEnumCCStateStunVictimGameplay.Forward
	local bRight 	= eChosenDirection == Unit.CodeEnumCCStateStunVictimGameplay.Right
	local bDown 	= eChosenDirection == Unit.CodeEnumCCStateStunVictimGameplay.Backward

	self:ShowControls(true)
	
	self:RemoveAllArrowPixies()
	if self.displayArrows == true then
		if bLeft == true then
			self:DrawArrowPixie(self.wndButtonLeft, 270)
		elseif bUp == true then
			self:DrawArrowPixie(self.wndButtonUp, 0)
		elseif bRight == true then
			self:DrawArrowPixie(self.wndButtonRight, 90)
		elseif bDown == true then
			self:DrawArrowPixie(self.wndButtonDown, 180)
		end
		
		self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):SetText("")
		self.wndButtonUp:FindChild("ProgressButtonArtUp"):SetText("")
		self.wndButtonRight:FindChild("ProgressButtonArtRight"):SetText("")
		self.wndButtonDown:FindChild("ProgressButtonArtDown"):SetText("")
	else
		self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):SetText(bLeft and GameLib.GetKeyBinding("StrafeLeft") or "")
		self.wndButtonUp:FindChild("ProgressButtonArtUp"):SetText(bUp and GameLib.GetKeyBinding("MoveForward") or "")
		self.wndButtonRight:FindChild("ProgressButtonArtRight"):SetText(bRight and GameLib.GetKeyBinding("StrafeRight") or "")
		self.wndButtonDown:FindChild("ProgressButtonArtDown"):SetText(bDown and GameLib.GetKeyBinding("MoveBackward") or "")
	end

	self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):Enable(bLeft)
	self.wndButtonUp:FindChild("ProgressButtonArtUp"):Enable(bUp)
	self.wndButtonRight:FindChild("ProgressButtonArtRight"):Enable(bRight)
	self.wndButtonDown:FindChild("ProgressButtonArtDown"):Enable(bDown)
	
	if self.playSounds == true then
		Sound.Play(166)
	end
	
	if not bLeft and not bUp and not bRight and not bDown then
		-- Error case:
		self:OnRemoveCCStateStun()
		return
	end

	self:OnCalculateTimeRemaining()
end

function Breakout:OnRemoveCCStateStun()
	self:ShowControls(false)
end

function Breakout:OnUpdateCCStateStun(fProgress) -- Updates Progress Bar
	if not self.wndProgressBar or not self.wndProgressBar:IsValid() then
		return
	end

	if self.wndProgressBar:FindChild("ProgressBar") then
		self.wndProgressBar:FindChild("ProgressBar"):SetMax(100)
		self.wndProgressBar:FindChild("ProgressBar"):SetFloor(0)
		self.wndProgressBar:FindChild("ProgressBar"):SetProgress(fProgress * 100)
	end

	self:OnCalculateTimeRemaining()
end

function Breakout:OnCalculateTimeRemaining()
	local nTimeRemaining = GameLib.GetCCStateStunTimeRemaining()
	if not nTimeRemaining or nTimeRemaining == 0 then
		if self.wndTimerBar and self.wndTimerBar:IsValid() then
			Apollo.CreateTimer("CalculateTimeRemaining", 0.1, false) -- Try again, in case it hasn't initialized yet
		end
		return
	end

	if self.wndTimerBar and self.wndTimerBar:IsShown() and self.wndTimerBar:FindChild("TimeRemainingContainer") then
		self.wndTimerBar:FindChild("TimeRemainingContainer"):Show(true)
		local nMaxTime = self.wndTimerBar:FindChild("TimeRemainingBar"):GetData()
		if not nMaxTime or nTimeRemaining > nMaxTime then
			nMaxTime = nTimeRemaining
			self.wndTimerBar:FindChild("TimeRemainingBar"):SetMax(100)
			self.wndTimerBar:FindChild("TimeRemainingBar"):SetData(nMaxTime)
			self.wndTimerBar:FindChild("TimeRemainingBar"):SetProgress(100)
		end
		self.wndTimerBar:FindChild("TimeRemainingBar"):SetProgress(math.min(math.max(nTimeRemaining / nMaxTime * 100, 0), 100), 50) -- 2nd Arg is the rate
	end

	if nTimeRemaining > 0 then
		Apollo.CreateTimer("CalculateTimeRemaining", 0.1, false)
	end
end

function Breakout:OnStunVGPressed(bPushed)
	self.wndButtonLeft:FindChild("ProgressButtonArtLeft"):SetCheck(bPushed)
	self.wndButtonUp:FindChild("ProgressButtonArtUp"):SetCheck(bPushed)
	self.wndButtonDown:FindChild("ProgressButtonArtDown"):SetCheck(bPushed)
	self.wndButtonRight:FindChild("ProgressButtonArtRight"):SetCheck(bPushed)
end

function Breakout:DrawArrowPixie(component, rotation)
	local loc = {
		fPoints = {0.5, 0.5, 0.5, 0.5},
		nOffsets = {-14, -14, 14, 14}
	}
	
	component:AddPixie({
	iLayer = 2,
	bLine = false,
	strSprite = "BreakoutSprites:Arrow",
	fRotation = rotation,
	loc = loc})
end

function Breakout:RemoveAllArrowPixies()
	self.wndButtonLeft:DestroyAllPixies()
	self.wndButtonUp:DestroyAllPixies()
	self.wndButtonRight:DestroyAllPixies()
	self.wndButtonDown:DestroyAllPixies()
end

---------------------------------------------------------------------------------------------------
-- Options
---------------------------------------------------------------------------------------------------

function Breakout:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

	local locUp = self.wndButtonUp and self.wndButtonUp:GetLocation() or self.savedUpLoc
	local locDown = self.wndButtonDown and self.wndButtonDown:GetLocation() or self.savedDownLoc
	local locLeft = self.wndButtonLeft and self.wndButtonLeft:GetLocation() or self.savedLeftLoc
	local locRight = self.wndButtonRight and self.wndButtonRight:GetLocation() or self.savedRightLoc
	local locProgress = self.wndProgressBar and self.wndProgressBar:GetLocation() or self.savedProgressLoc
	local locTimer = self.wndTimerBar and self.wndTimerBar:GetLocation() or self.savedTimerLoc
	
	local tSaved =
	{
		tDisplayArrows = self.displayArrows,
		tPlaySounds = self.playSounds,
		tUpButtonLoc = locUp and locUp:ToTable() or nil,
		tDownButtonLoc = locDown and locDown:ToTable() or nil,
		tLeftButtonLoc = locLeft and locLeft:ToTable() or nil,
		tRightButtonLoc = locRight and locRight:ToTable() or nil,
		tProgressLoc = locProgress and locProgress:ToTable() or nil,
		tTimerLoc = locTimer and locTimer:ToTable() or nil,
	}

	return tSaved
end

function Breakout:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character and tSavedData then
		self.displayArrows = tSavedData.tDisplayArrows
		self.playSounds = tSavedData.tPlaySounds
		self.savedUpLoc = WindowLocation.new(tSavedData.tUpButtonLoc)
		self.savedDownLoc = WindowLocation.new(tSavedData.tDownButtonLoc)
		self.savedLeftLoc = WindowLocation.new(tSavedData.tLeftButtonLoc)
		self.savedRightLoc = WindowLocation.new(tSavedData.tRightButtonLoc)
		self.savedProgressLoc = WindowLocation.new(tSavedData.tProgressLoc)
		self.savedTimerLoc = WindowLocation.new(tSavedData.tTimerLoc)
	end
end

---------------------------------------------------------------------------------------------------
-- Instance
---------------------------------------------------------------------------------------------------

local BreakoutInst = Breakout:new()
BreakoutInst:Init()
